#!/mnt/syb8634/server/php5-cgi -qd register_argc_argv=1
<?php
/*
 * utils.php - Various functions for mono.php
 * By: Erik Bernhardson
 * Date: 2-20-2009
 */

// escapeshellarg() turns UTF-8 encoded strings into blank strings
// this prevents that
function my_escapeshellarg($string) {
  $string = str_replace("'", "'\\''", $string);
  return "'$string'";
}

// Encodes a query string with multiple options
function rawurlencodeQuery($query) {
  $tmp = explode('&', $query);
  foreach($tmp as $key => $item) {
    // @ to suppress errors when no = in $item
    @list($opt, $val) = explode('=', $item, 2);
    $tmp[$key] = rawurlencode($opt);
    if(isset($val)) {
      $tmp[$key] .= '='.rawurlencode($val);
    }
    unset($opt, $val);
  }
  $output = implode('&', $tmp);
  return $output;
}

/* m3u2jsp and pls2jsp origionally by Robert Widdick */
function playlist2jsp($infile) {
  _log(__FUNCTION__);

  $ext = strtolower(pathinfo($infile, PATHINFO_EXTENSION));

  // Only accept extensions we have classes for
  // and dont parse jsp's
  if($ext == 'jsp' || !file_exists(SYSPATH."/php/$ext.class.php"))
    return $infile;

  // Set the outfile to the infile with a new extension
  $outfile = substr($infile, 0, -strlen($ext)).'jsp';

  // Wrapped in do while(FALSE) loop to allow break
  // Only make the playlist once if possible
  do if(!file_exists($outfile)) {
    $dir = dirname(realpath($outfile));

    if(!is_writable($dir)) {
      $outfile = '/tmp/playlist-'.md5($infile);
      if(file_exists($outfile))
        break;
    }

    _log('creating playlist');

    require_once('php/jsp.class.php');
    $jsp = new jsp();
    require_once("php/$ext.class.php");
    $playlist = new $ext($infile);

    if(count($playlist->entries) == 0) {
      _log("Failed loading $infile with $ext.");
      return $infile;
    }

    $jsp->setBasePath($dir);
    foreach($playlist->entries as $entry)
      $jsp->addFile($entry['File'], $entry['Title']);
    file_put_contents($outfile, $jsp->getPlaylist());
  } while(FALSE);

  _log('running playlist: '.$outfile);
  return $outfile;
}

function getServLinkByName($shareName) {
  // Import the stored NMT flash settings
  $settings = file_get_contents('/tmp/setting.txt');

  // Can this be done faster without regular expressions?
  // Find the defined shares
  preg_match_all('/servname([0-9]+)=(.*)/i', $settings, $regs);
  
  // Loop through the names until we find the matching name
  foreach($regs[2] as $key => $servname)
    if(strcasecmp($shareName,$servname) === 0) {
      // Search for the matching servlink
      if(preg_match('/servlink'.$regs[1][$key].'=(.*)/i', $settings, $tmp))
        return $tmp[1]; // Success
      else _log('found servname'.$regs[1][$key].' but no matching servlink');
    }

  // No matching share was found
  _log('possible shares: '.implode(' ', $regs[2]));
  return False;
}

//         1    2  3   4        5                6             7    8
// sample: file:///opt/sybhttpd/localhost.drives/NETWORK_SHARE/vids/foobar.mkv
function mountFilesystemByPath($inputPath) {
  _log(__FUNCTION__.': '.$inputPath);
  $path = explode('/', $inputPath);

  if(isset($path[6])) {
    if($path[6] == 'NETWORK_SHARE') {
      mountFilesystemByName($path[7]);
      if(alreadyMounted($path[7])) {
        if(file_exists($inputPath)) {
          _log('Mount Success.');
        } else _log('Share mounted but file does not exist.');
      } else _log('Mount Failed.');
    } else _log('Not a Network Share');
  } else _log('Path not recognized');
}

function alreadyMounted($shareName) {
  // The mounts file preserves whitespace seperation by using \040 instead
  // of a space in the variable parts of its output, so adjust the shareName to match
  $shareName = str_replace(' ', '\040', $shareName);
  // must start and finish with spaces to prevent getting wrong share
  $lookingFor = ' /opt/sybhttpd/localhost.drives/NETWORK_SHARE/'.$shareName.' ';
  $return = (stripos(file_get_contents('/proc/mounts'), $lookingFor) !== False);
  if($return === False) {
    _log("Couldn't find{$lookingFor}in /proc/mounts");
  }
  return $return;
}

function mountFilesystemByName($shareName) {
  _log(__FUNCTION__.': '.$shareName);

  if(alreadyMounted($shareName)) {
    _log("$shareName already mounted");
    return;
  }

  $servLink = getServLinkByName($shareName);

  if($servLink !== False) {
    // rawurlencode used because smbclient.cgi only accepts %20 not + as a space
    // servlink has multiple query options in it
    $query = 'smb.cmd=mount'.
            '&smb.opt='.rawurlencodeQuery($servLink).
            '&smb.name='.rawurlencode($shareName);
    _log('performing mount: '.$query);
    // Pass the mounting on to smbclient.cgi
    if(IS_NMT) {
      if(REVERT_UDN) {
        preg_match('/udn=(.*)/', file_get_contents('/tmp/setting.txt'), $regs);
        $udn = $regs[1];
      }
      exec('/opt/sybhttpd/default/smbclient.cgi '.my_escapeshellarg($query));
      if(REVERT_UDN) {
        if(!empty($udn)) {
          exec('pflash set udn '.my_escapeshellarg($udn));
        } else _log('Tried to revert udn, but none found.');
      }
    } else _log('Not an NMT, so not mounting.');
  } else _log('no matching share');
}

  
function executeMono($args) {
	execute('/bin/mono.real', $args);
	return;
}

function executeMount($args) {
  _log(__FUNCTION__);
  execute('/bin/busybox mount', $args, TRUE);
  return;
}

function execute($cmd, $args, $passthru = False) {
    _log(__FUNCTION__." $cmd $args");

  // Remove args[0] since its the path to this script
  array_shift($args);

  foreach($args as $key => $arg)
    $args[$key] = my_escapeshellarg($arg);

  $cmd = $cmd.' '.implode(' ', $args);
  _log($cmd);
  // Allows for testing on machines other than NMT
  if(IS_NMT) {
    _log("Starting $cmd");
		if($passthru) {
			passthru($cmd);
		} else {
	    exec($cmd, $output, $return);
      _log("$cmd Exited with return status of $return");
		}
  } else _log("Test Run, not executing $cmd");
}
